class DummyProtocolData:
    def __init__(self, id, type, data, subType: list = None):
        self.id = id
        self.type = type
        self.data = data
        self.subType = []
        if subType is not None:
            for _subType in subType:
                self.addSubType(_subType)

    def addSubType(self, type):
        self.subType.append(type)

    def __repr__(self):
        L = ["%s=%r" % (key, value) for key, value in self.__dict__.items()]
        return "%s(%s)" % (self.__class__.__name__, ", ".join(L))


class DummyProtocol:
    def __init__(self, protocol: int = 5, data: DummyProtocolData = None):
        self.protocol = protocol
        self.data = data

    def __repr__(self):
        L = ["%s=%r" % (key, value) for key, value in self.__dict__.items()]
        return "%s(%s)" % (self.__class__.__name__, ", ".join(L))


class DummyThrift:
    def __init__(self, name: str = None, **kwargs):
        if name is not None:
            self.__name__ = name
        if kwargs:
            for key in kwargs:
                setattr(self, key, kwargs[key])

    def __repr__(self):
        L = ["%s=%r" % (key, value) for key, value in self.__dict__.items()]
        return "%s(%s)" % (self.__class__.__name__, ", ".join(L))


class DummyProtocolSerializer:
    def __init__(self, instance: any, name: str, data: list, protocol: int):
        self.instance = instance
        self.name = name
        self.data = data
        self.protocol = protocol

    def __bytes__(self):
        """Convert to proto data."""
        data = []
        instance = self.instance
        protocol = self.protocol
        if protocol == 3:
            data = [128, 1, 0, 1] + instance.getStringBytes(self.name) + [0, 0, 0, 0]
        elif protocol in [4, 5]:
            protocol = 4
            data = [130, 33, 0] + instance.getStringBytes(self.name, isCompact=True)
        else:
            raise ValueError(f"Unknower protocol: {protocol}")
        data += instance.generateDummyProtocolField(self.data, protocol) + [0]
        return bytes(data)

    def __repr__(self):
        L = ["%s=%r" % (key, value) for key, value in self.__dict__.items()]
        return "%s(%s)" % (self.__class__.__name__, ", ".join(L))



import time
import threading
import struct

from ..client import CHRLINE
from ..services import *


class ConnManager(object):
    def __init__(self, line_client: CHRLINE):
        self.line_client = line_client
        self.conns = []
        self.hook_callback = None
        self.curr_ping_id = 0
        self.subscriptionIds = {}
        self.SignOnRequests = {}
        self.OnPingCallback = self._OnPingCallback
        self.OnSignReqResp = {}
        self.OnSignOnResponse = self._OnSignOnResponse
        self.OnPushResponse = self._OnPushResponse

        self._eventSynced = False
        self._revisionSynced = False
        self._pingInterval = 30
        self._access_token = None
    
    @property
    def accessToken(self):
        return self._access_token
    
    @accessToken.setter
    def accessToken(self, newToken):
        self._access_token = newToken

    def initializeConn(self, state: int = 1):
        """Create new conn and return it."""
        from .conn import Conn

        _conn = Conn(self)
        if state == 1:
            self.conns.append(_conn)
            self.accessToken = self.line_client.authToken
        tosend_headers = {
            "x-line-application": self.line_client.server.Headers["x-line-application"],
            "x-line-access": self.accessToken,
        }
        _conn.new("gw.line.naver.jp", 443, "/PUSH/1/subs?m=20", tosend_headers)
        return _conn

    def InitAndRead(self, initServices: list = [3, 5]):
        """Init conn state and read data."""
        if not self.conns:
            raise ValueError("No valid connections found.")
        cl = self.line_client
        _conn = self.conns[0]
        FLAG = 0
        _conn.wirteRequest(0, bytes([0, FLAG, self._pingInterval]))
        cl.log(f"[PUSH] send status frame. flag:{FLAG}, pi:{self._pingInterval}")
        for service in initServices:
            cl.log(f"[PUSH] Init service: {service}")
            ex_val = {}
            if service == 3:
                subscriptionId = int(time.time() * 1000)
                syncToken = ""
                ex_val = {
                    "subscriptionId": subscriptionId,
                    "syncToken": syncToken,
                }
                cl.log(
                    f"[SQ_FETCHER][SQ] request fetchMyEvent({subscriptionId}), syncToken:{syncToken}"
                )
            elif service == 5:
                ex_val = {
                    "revision": self.line_client.revision,
                    "count": 100,
                    "globalRev": cl.globalRev,
                    "individualRev": cl.individualRev,
                    "fullSyncRequestReason": None,
                    "lastPartialFullSyncs": None,
                }
            self.buildAndSendSignOnRequest(_conn, service, **ex_val)
        cl.log(f"[PUSH] CONN start read push.")
        _conn.read()
        cl.log(f"[PUSH] CONN died on PingId={self.curr_ping_id}")
        self.conns.remove(_conn)

    def SendAndReadSignOnRequest(
        self, serviceType, waitAndReadResp: bool = False, **kwargs
    ):
        """Test send request and read response."""
        if not self.conns:
            raise ValueError("No valid connections found.")
        # _conn = self.conns[0]
        conns = []
        for c in self.conns[1:]:
            if c.IsAble2Request():
                conns.append(c)
        if len(conns) < 1:
            pusher = ConnManager(self.line_client)
            pusher.initializeConn()
            # use thread for send pings
            _td = threading.Thread(target=pusher.InitAndRead, args=([],))
            _td.daemon = True
            _td.start()
            _conn = pusher.conns[0]
            self.conns.append(_conn)
        else:
            for c in conns:
                if not c._closed:
                    _conn = c
                    break
        _, reqId = self.buildAndSendSignOnRequest(_conn, serviceType, **kwargs)
        if waitAndReadResp:
            raise NotImplementedError()

            def Callback(self, reqId, data):
                self.OnSignReqResp[reqId] = data

            self.SignOnRequests[reqId][2] = Callback
            return self.OnSignReqResp[reqId]

    def buildRequest(self, service, data):
        """Build request struct."""
        return struct.pack("!H", len(data)) + bytes([service]) + data

    def buildAndSendSignOnRequest(self, conn: any, serviceType: int, **kwargs):
        """
        Build and send sign-on-request.

        ServiceType:
            - 3: fetchMyEvents
            - 5: fecthOps / sync
        """
        cl = self.line_client
        _id = len(self.SignOnRequests) + 1
        _payload = struct.pack("!H", _id)
        _req = None
        serviceName = None
        methodName = None
        if serviceType == 1:
            raise NotImplementedError
        elif serviceType == 3:
            serviceName = "Square"
            methodName = "fetchMyEvents"
            _req = self.buildFetchMyEventsRequest(**kwargs)
        elif serviceType == 5:
            serviceName = "Talk"
            methodName = "fetchOps"
            if cl.DEVICE_TYPE in cl.SYNC_SUPPORT:
                methodName = "sync"
                _req = self.buildServiceRequest(serviceName, methodName, **kwargs)
            else:
                _req = self.buildFetchOpsRequest(**kwargs)
        elif serviceType == 6:
            raise NotImplementedError
        elif serviceType == "sendMessage":
            raise NotImplementedError
            methodName = "sendMessage"
            serviceType = 5
            _req = self.buildSendMessageRequest(**kwargs)
        else:
            raise ValueError(f"unknow serviceType: {serviceType}")
        _payload += bytes([serviceType, 0])
        _payload += struct.pack("!H", len(_req))
        _payload += _req
        self.SignOnRequests[_id] = [serviceType, methodName, None]
        self.line_client.log(
            f"[H2][PUSH] send sign-on-request. requestId:{_id}, service:{serviceType}",
            True,
        )
        conn.wirteRequest(2, _payload)
        return _payload, _id

    def buildFetchMyEventsRequest(self, subscriptionId, syncToken):
        """Build request for fetchMyEvents."""
        cl = self.line_client
        params = [
            [
                12,
                1,
                [
                    [10, 1, subscriptionId],
                    [11, 2, syncToken],
                    [8, 3, 100],
                ],
            ]
        ]
        return bytes(cl.generateDummyProtocol("fetchMyEvents", params, 4))

    def buildFetchOpsRequest(self, revision: int = -1, **kargs):
        """Build request for fetchOps."""
        cl = self.line_client
        if revision == -1:
            revision = cl.revision
        params = [
            [10, 2, revision],
            [8, 3, 200],  # use 200.
            [10, 4, cl.globalRev],
            [10, 5, cl.individualRev],
        ]
        return bytes(cl.generateDummyProtocol("fetchOps", params, 4))

    def buildServiceRequest(self, serviceName: str, methodName: str, **kargs):
        """Build request for service."""
        cl = self.line_client
        service = f"{serviceName}Service"
        serviceStruct = f"{service}.{service}Struct"
        methodRequest = methodName.title() + "Request"
        unitCalled = f"{serviceStruct}.{methodRequest}"
        ins = eval(f"{unitCalled}")
        payload = ins(**kargs)
        data = cl.generateDummyProtocol(methodName, payload, 4)
        return bytes(data)

    def _OnSignOnResponse(self, reqId, isFin, data):
        if reqId in self.SignOnRequests:
            serviceType = self.SignOnRequests[reqId][0]
            methodName = self.SignOnRequests[reqId][1]
            callback = self.SignOnRequests[reqId][2]
            serviceName = "TalkService"
            cl = self.line_client
            cl.log(
                f"[PUSH] receives sign-on-response frame. requestId:{reqId}, service:{serviceType}, isFin:{isFin}, payload:{data[:20].hex()}",
                True,
            )
            if serviceType == 3:
                data = cl.TCompactProtocol(cl, data)
                resp = data.res
                if "error" in resp:
                    cl.log(
                        f"[SQ_FETCHER][SQ] can't use PUSH for OpenChat:{resp['error']}"
                    )
                    return False
                subscription = cl.checkAndGetValue(resp, "subscription", 1)
                events = cl.checkAndGetValue(resp, "events", 2)
                syncToken = cl.checkAndGetValue(resp, "syncToken", 3)
                subscriptionId = cl.checkAndGetValue(subscription, "subscriptionId", 1)
                cl.log(
                    f"[SQ_FETCHER][SQ] response fetchMyEvent({subscriptionId}) events:{len(events)}, syncToken:{syncToken}"
                )
                cl.subscriptionId = subscriptionId
                if subscriptionId is not None:
                    self.subscriptionIds[subscriptionId] = time.time()
                if not self._eventSynced:
                    cl.setEventSyncToken(syncToken)
                    cl.log(
                        f"[SQ_FETCHER][SQ] myEvents start({subscriptionId}) : syncToken:{cl.eventSyncToken}"
                    )
            elif serviceType == 5:
                _conn = self.conns[0]
                try:
                    data = cl.TMoreCompactProtocol(cl, data)
                    resp = data.res
                    if methodName == "sync":
                        serviceName = "SyncService"
                    if cl.use_thrift:
                        resp = cl.serializeDummyProtocolToThrift(
                            data.dummyProtocol, readWith=f"{serviceName}.{methodName}"
                        )
                    if methodName == "sync":
                        ops = resp
                        sht, shd = cl.talk_handler.SyncHandler(resp)
                        ex_val = {
                            "revision": cl.globalRev,
                            "count": 100,
                            "globalRev": cl.globalRev,
                            "individualRev": cl.individualRev,
                            "fullSyncRequestReason": None,
                            "lastPartialFullSyncs": None,
                        }
                        if sht == 1:
                            ops = shd
                            cl.log(
                                f"[POLLING][PUSH] response sync. operations:{len(ops)}",
                                True,
                            )
                            for op in ops:
                                self.hook_callback(cl, serviceType, op)
                                revision = cl.checkAndGetValue(op, "revision", 1)
                                cl.setRevision(
                                    revision
                                )
                        elif sht == 2:
                            cl.setRevision(shd)
                        else:
                            raise RuntimeError
                        ex_val["revision"] = cl.revision
                        self.buildAndSendSignOnRequest(_conn, serviceType, **ex_val)
                        return
                    elif methodName == "fetchOps":
                        ops = resp
                        cl.log(
                            f"[POLLING][PUSH] response fetchOps. operations:{len(ops)}",
                            True,
                        )
                        for op in ops:
                            opType = cl.checkAndGetValue(op, "type", 3)
                            param1 = cl.checkAndGetValue(op, "param1", 10)
                            param2 = cl.checkAndGetValue(op, "param2", 11)
                            if opType == 0:
                                if param1 is not None:
                                    cl.individualRev = param1.split("\x1e")[0]
                                    cl.log(f"individualRev: {cl.individualRev}", True)
                                if param2 is not None:
                                    cl.globalRev = param2.split("\x1e")[0]
                                    cl.log(f"globalRev: {cl.globalRev}", True)
                            cl.setRevision(cl.checkAndGetValue(op, "revision", 1))
                            self.hook_callback(cl, serviceType, op)
                        # LOOP
                        fetch_req_data = {"revision": cl.revision}
                        self.buildAndSendSignOnRequest(_conn, serviceType, **fetch_req_data)
                    else:
                        # TODO:
                        # Callback resp to ReqId
                        if callback is not None:
                            callback(self, reqId, resp)
                except Exception as e:
                    raise Exception(f"[PUSH] response {methodName} error: {e}")
            else:
                raise ValueError(
                    f"[PUSH] receives invalid sign-on-response frame. requestId:{reqId}, service:{serviceType}"
                )

    def _OnPushResponse(self, serviceType, pushId, pushPayload):
        cl = self.line_client
        tcp = cl.TCompactProtocol(cl, passProtocol=True)
        tcp.data = pushPayload
        subscriptionId = tcp.x(False)[1]
        cl.log(
            f"[PUSH] id:{pushId}, service:{serviceType}, subscriptionId:{subscriptionId}",
            True,
        )
        if serviceType == 3:
            for event in self.line_client._Poll__fetchMyEvents():
                _type = cl.checkAndGetValue(event, "type", 3)
                cl.log(
                    f"[SQ_FETCHER][PUSH] subscriptionId:{subscriptionId}, eventType:{_type}",
                    True,
                )
                self.hook_callback(self.line_client, serviceType, event)
        else:
            raise NotImplementedError(f"Not support type: {serviceType}")

    def _OnPingCallback(self, pingId):
        cl = self.line_client
        self.curr_ping_id = pingId

        # check subscriptionIds need refresh.
        # LOG:
        #   - 221227: change use ts.
        refreshIds = []
        t1 = time.time()
        for subscriptionId in self.subscriptionIds.keys():
            t2 = self.subscriptionIds[subscriptionId]
            if (t1 - t2) >= 3600:
                self.subscriptionIds[subscriptionId] = time.time()
                refreshIds.append(subscriptionId)
        if refreshIds:
            cl.log(f"[SQ_FETCHER][PUSH] refresh subscriptionId: {refreshIds}")
            cl.refreshSquareSubscriptions(refreshIds)

        # when using the PUSH endpoint, you may not be able to update the AuthToken properly.
        # because you can't get the `x-line-next-access` header on the socket connection.
        # here use http protocol to check.
        if pingId % 3 == 0:
            cl.noop()
            oldToken = self.accessToken
            newToken = self.line_client.authToken
            if oldToken != newToken:
                cl.log(f"[PUSH] renew push conn for new authToken...")
                self.accessToken = newToken
                self.conns[0].close()




# -*- coding: utf-8 -*-

class PrimaryQrCodeMigrationLongPollingService(object):
    PQCMLPS_REQ_TYPE = 4
    PQCMLPS_RES_TYPE = 4
    PQCMLPS_API_PATH = "/EXT/auth/feature-user/lp/api/primary/mig/qr"

    def __init__(self):
        pass
        
    def checkIfEncryptedE2EEKeyReceived(self, sessionId: str, newDevicePublicKey: bytes, encryptedQrIdentifier: str):
        params = [
            [11, 1, sessionId],
            [12, 2, [
                [11, 1, newDevicePublicKey],
                [11, 2, encryptedQrIdentifier]
            ]]
        ]
        params = [
            [12, 1, params]
        ]
        sqrd = self.generateDummyProtocol(
            'checkIfEncryptedE2EEKeyReceived', params, self.PQCMLPS_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(
            self.PQCMLPS_API_PATH ,sqrd, self.PQCMLPS_RES_TYPE, access_token=sessionId)


__all__ = ['ttypes', 'constants', 'TalkService', 'E2EEKeyBackupService', 'AccessTokenRefreshService', 'BuddyService', 'CallService', 'SyncService', 'HomeSafetyCheckService', 'SquareService', 'SquareLiveTalkService', 'SecondaryQrCodeLoginService']



# -*- coding: utf-8 -*-
"""
Automatically generated by DeachSword-DearSakura
Author: YinMo (WEDeach)
Version: 1.0.6
"""
from typing import List
from .BaseService import BaseService


class RelationService(BaseService):
    RelationService_REQ_TYPE = 4  # BASE_VAL
    RelationService_RES_TYPE = 4  # BASE_VAL
    RelationService_API_PATH = None  # BASE_VAL

    def __init__(self):
        self.RelationService_API_PATH = "/RE4"

    def getTargetProfiles(self, mids: List[str]):
        """
        Get target profiles (test).

        GENERATED BY YinMo0913_DeachSword-DearSakura_v1.0.6.py
        DATETIME: 03/10/2023, 13:41:11
        """
        METHOD_NAME = "getTargetProfiles"
        targets = []
        for mid in mids:
            targets.append(
                [
                    [11, 1, mid]
                ]
            )
        request = [
            [15, 1, [12, targets]]
        ]
        params = [
            [12, 1, request]
        ]
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )

    def getUserStatuses(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!

        GENERATED BY YinMo0913_DeachSword-DearSakura_v1.0.6.py
        DATETIME: 03/10/2023, 13:41:11
        """
        raise Exception("getUserStatuses is not implemented")
        METHOD_NAME = "getUserStatuses"
        params = []
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )

    def getRecommendationDetails(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!

        GENERATED BY YinMo0913_DeachSword-DearSakura_v1.0.6.py
        DATETIME: 03/10/2023, 13:41:11
        """
        raise Exception("getRecommendationDetails is not implemented")
        METHOD_NAME = "getRecommendationDetails"
        params = []
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )

    def getContactCalendarEvents(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!

        GENERATED BY YinMo0913_DeachSword-DearSakura_v1.0.6.py
        DATETIME: 03/10/2023, 13:41:11
        """
        raise Exception("getContactCalendarEvents is not implemented")
        METHOD_NAME = "getContactCalendarEvents"
        params = []
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )

    def getBlockDetails(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!

        GENERATED BY YinMo0913_DeachSword-DearSakura_v1.0.6.py
        DATETIME: 03/10/2023, 13:41:11
        """
        raise Exception("getBlockDetails is not implemented")
        METHOD_NAME = "getBlockDetails"
        params = []
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )

    def getContactsV3(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!

        GENERATED BY YinMo0913_DeachSword-DearSakura_v1.0.6.py
        DATETIME: 03/10/2023, 13:41:11
        """
        raise Exception("getContactsV3 is not implemented")
        METHOD_NAME = "getContactsV3"
        params = []
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )

    def getFriendDetails(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!

        GENERATED BY YinMo0913_DeachSword-DearSakura_v1.0.6.py
        DATETIME: 03/10/2023, 13:41:11
        """
        raise Exception("getFriendDetails is not implemented")
        METHOD_NAME = "getFriendDetails"
        params = []
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )

    def addFriendByMid(
        self,
        userMid: str,
        reference: str = None,
        trackingMetaType: int = 5,
        trackingMetaHint: str = None,
    ):
        """Add friend by mid."""
        METHOD_NAME = "addFriendByMid"
        addMeta = [[11, 1, trackingMetaHint]]
        trackingMeta = [[12, trackingMetaType, addMeta]]
        tracking = [[11, 1, reference], [12, 2, trackingMeta]]
        request = [[8, 1, self.getCurrReqId()], [11, 2, userMid], [12, 3, tracking]]
        params = [[12, 1, request]]
        sqrd = self.generateDummyProtocol(
            METHOD_NAME, params, self.RelationService_REQ_TYPE
        )
        return self.postPackDataAndGetUnpackRespData(
            self.RelationService_API_PATH, sqrd, self.RelationService_RES_TYPE
        )



# -*- coding: utf-8 -*-


class ChrlineException(Exception):
    pass


class LineServiceException(ChrlineException):
    def __init__(self, data: dict, code=None, reason=None, parameterMap=None, raw=None):
        self.code = data.get("code", -519)  # base code for CHR
        self.message = str(data.get("message", ""))
        self.metadata = data.get("metadata", None)
        self.raw = data.get("raw", data)

        if code is not None:
            self.code = code
        if reason is not None:
            self.message = str(reason)
        if parameterMap is not None:
            self.metadata = parameterMap
        if raw is not None:
            self.raw = raw

        fmt = "Code: {0}"
        if self.message is not None and len(self.message):
            fmt += ", Message: {1}"
        if self.metadata is not None:
            fmt += ", Metadata: {2}"

        super().__init__(fmt.format(self.code, self.message, self.metadata))



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
import logging
from .ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
all_structs = []


class Iface(object):
    def refresh(self, request):
        """
        Parameters:
         - request

        """
        pass


class Client(Iface):
    def __init__(self, iprot, oprot=None):
        self._iprot = self._oprot = iprot
        if oprot is not None:
            self._oprot = oprot
        self._seqid = 0

    def refresh(self, request):
        """
        Parameters:
         - request

        """
        self.send_refresh(request)
        return self.recv_refresh()

    def send_refresh(self, request):
        self._oprot.writeMessageBegin('refresh', TMessageType.CALL, self._seqid)
        args = refresh_args()
        args.request = request
        args.write(self._oprot)
        self._oprot.writeMessageEnd()
        self._oprot.trans.flush()

    def recv_refresh(self):
        iprot = self._iprot
        (fname, mtype, rseqid) = iprot.readMessageBegin()
        if mtype == TMessageType.EXCEPTION:
            x = TApplicationException()
            x.read(iprot)
            iprot.readMessageEnd()
            raise x
        result = refresh_result()
        result.read(iprot)
        iprot.readMessageEnd()
        if result.success is not None:
            return result.success
        if result.e is not None:
            raise result.e
        raise TApplicationException(TApplicationException.MISSING_RESULT, "refresh failed: unknown result")


class Processor(Iface, TProcessor):
    def __init__(self, handler):
        self._handler = handler
        self._processMap = {}
        self._processMap["refresh"] = Processor.process_refresh
        self._on_message_begin = None

    def on_message_begin(self, func):
        self._on_message_begin = func

    def process(self, iprot, oprot):
        (name, type, seqid) = iprot.readMessageBegin()
        if self._on_message_begin:
            self._on_message_begin(name, type, seqid)
        if name not in self._processMap:
            iprot.skip(TType.STRUCT)
            iprot.readMessageEnd()
            x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
            oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
            x.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()
            return
        else:
            self._processMap[name](self, seqid, iprot, oprot)
        return True

    def process_refresh(self, seqid, iprot, oprot):
        args = refresh_args()
        args.read(iprot)
        iprot.readMessageEnd()
        result = refresh_result()
        try:
            result.success = self._handler.refresh(args.request)
            msg_type = TMessageType.REPLY
        except TTransport.TTransportException:
            raise
        except AccessTokenRefreshException as e:
            msg_type = TMessageType.REPLY
            result.e = e
        except TApplicationException as ex:
            logging.exception('TApplication exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = ex
        except Exception:
            logging.exception('Unexpected exception in handler')
            msg_type = TMessageType.EXCEPTION
            result = TApplicationException(TApplicationException.INTERNAL_ERROR, 'Internal error')
        oprot.writeMessageBegin("refresh", msg_type, seqid)
        result.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

# HELPER FUNCTIONS AND STRUCTURES


class refresh_args(object):
    """
    Attributes:
     - request

    """


    def __init__(self, request=None,):
        self.request = request

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.request = RefreshAccessTokenRequest()
                    self.request.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('refresh_args')
        if self.request is not None:
            oprot.writeFieldBegin('request', TType.STRUCT, 1)
            self.request.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(refresh_args)
refresh_args.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'request', [RefreshAccessTokenRequest, None], None, ),  # 1
)


class refresh_result(object):
    """
    Attributes:
     - success
     - e

    """


    def __init__(self, success=None, e=None,):
        self.success = success
        self.e = e

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 0:
                if ftype == TType.STRUCT:
                    self.success = RefreshAccessTokenResponse()
                    self.success.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 1:
                if ftype == TType.STRUCT:
                    self.e = AccessTokenRefreshException.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('refresh_result')
        if self.success is not None:
            oprot.writeFieldBegin('success', TType.STRUCT, 0)
            self.success.write(oprot)
            oprot.writeFieldEnd()
        if self.e is not None:
            oprot.writeFieldBegin('e', TType.STRUCT, 1)
            self.e.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(refresh_result)
refresh_result.thrift_spec = (
    (0, TType.STRUCT, 'success', [RefreshAccessTokenResponse, None], None, ),  # 0
    (1, TType.STRUCT, 'e', [AccessTokenRefreshException, None], None, ),  # 1
)
fix_spec(all_structs)
del all_structs



import time
import socket
import ssl
import struct
import h2.connection

from .connManager import ConnManager


class Conn(object):
    def __init__(self, manager: ConnManager):
        self.manager = manager
        self.conn = None
        self.writer = None
        self.h2_headers = []
        self.is_not_finished = False
        self.cache_data = b""
        self.notFinPayloads = {}

        self._last_send_time = 0
        self._closed = False

    @property
    def client(self):
        return self.manager.line_client

    def new(self, host, port, path, headers: dict = {}):
        ctx = ssl.create_default_context()
        ctx.set_alpn_protocols(["h2"])
        s = socket.create_connection((host, port))
        self.writer = ctx.wrap_socket(s, server_hostname=host)
        self.conn = h2.connection.H2Connection()
        self.h2_headers = [
            (":method", "POST"),
            (":authority", host),
            (":scheme", "https"),
            (":path", path),
        ]
        for h in headers.keys():
            self.h2_headers.append((h, headers[h]))
        self.conn.initiate_connection()
        self.conn.send_headers(1, self.h2_headers)
        self.send()

    def send(self):
        send_data = self.conn.data_to_send()
        if send_data:
            self.client.log(f"[H2][PUSH] send data: {send_data.hex()}", True)
            self.writer.sendall(send_data)
            self._last_send_time = time.time()

    def writeByte(self, data: bytes):
        self.conn.send_data(stream_id=1, data=data, end_stream=False)
        self.send()

    def wirteRequest(self, requestType: int, data: bytes):
        self.writeByte(self.manager.buildRequest(requestType, data))

    def read(self):
        try:
            response_stream_ended = self._closed
            self.send()
            while not response_stream_ended and self.client.is_login:
                data = self.writer.recv(65536 * 1024)
                if not data:
                    break
                events = self.conn.receive_data(data)
                for event in events:
                    if isinstance(event, h2.events.DataReceived):
                        # update flow control so the server doesn't starve us
                        self.conn.acknowledge_received_data(
                            event.flow_controlled_length, event.stream_id
                        )

                        _data = event.data
                        if len(_data) < 4:
                            self.client.log(f"[CONN] Invalid Packet: {_data.hex()}")
                            continue
                        self.onDataReceived(_data)
                    elif isinstance(event, h2.events.StreamEnded):
                        # response body completed, let's exit the loop
                        response_stream_ended = True
                        break
                    elif isinstance(event, h2.events.StreamReset):
                        raise RuntimeError("Stream reset: %d" % event.error_code)
                # send any pending data to the server
                self.send()
            self.conn.close_connection()
            self.send()
        except Exception as e:
            self.client.log(f"[CONN] task disconnect: {e}")
            if isinstance(e, OSError):
                pass
            else:
                raise e
        self._closed = True
        # close the socket
        self.writer.close()

    def IsAble2Request(self):
        if self.client.is_login and not self._closed:
            if time.time() - self._last_send_time > 0.5:
                return True
        return False

    def readPacketHeader(self, data):
        (_dl,) = struct.unpack("!H", data[:2])
        _dt = data[2]
        _dd = data[3:]
        return _dt, _dd, _dl

    def onDataReceived(self, data):
        # long data received
        if self.is_not_finished:
            data = self.cache_data + data

        # more response body data received
        self.client.log(f"[H2][PUSH] receives packet. raw:{data.hex()}", True)
        _dt, _dd, _dl = self.readPacketHeader(data)
        if _dl > len(_dd):
            self.is_not_finished = True
            self.cache_data = data
            return
        else:
            self.is_not_finished = False
            if len(_dd) > _dl:
                self.onPacketReceived(_dt, _dd[:_dl])
                data = _dd[_dl:]
                self.client.log(f"[PUSH] extra data {data.hex()[:50]}...", True)
                return self.onDataReceived(data)
        self.onPacketReceived(_dt, _dd)

    def onPacketReceived(self, _dt, _dd):
        if _dt == 1:
            _pingType = _dd[0]
            (_pingId,) = struct.unpack("!H", _dd[1:3])
            self.client.log(f"[PUSH] receives ping frame. pingId:{_pingId}", True)

            if _pingType == 2:
                self.writeByte(bytes([0, 3, 1, 1]) + struct.pack("!H", _pingId))
                self.client.log(f"[PUSH] send ping ack. pingId:{_pingId}", True)
                self.manager.OnPingCallback(_pingId)
            else:
                raise NotImplementedError(f"ping type not Implemented: {_pingType}")
        elif _dt == 3:
            (I,) = struct.unpack("!H", _dd[0:2])
            _requestId = I & 32767
            # Android using 32768, CHRLINE use (32768 / 2)
            _isFin = (I & 32768) != 0
            _responsePayload = _dd[2:]
            if _isFin:
                if _requestId in self.notFinPayloads:
                    _responsePayload = (
                        self.notFinPayloads[_requestId] + _responsePayload
                    )
                    del self.notFinPayloads[_requestId]
                self.manager.OnSignOnResponse(_requestId, _isFin, _responsePayload)
            else:
                self.client.log(
                    f"[PUSH] receives long data. requestId: {_requestId}, I={I}", True
                )
                if _requestId not in self.notFinPayloads:
                    self.notFinPayloads[_requestId] = b""
                self.notFinPayloads[_requestId] += _responsePayload
        elif _dt == 4:
            _pushType = _dd[0]
            _serviceType = _dd[1]
            (_pushId,) = struct.unpack("!i", _dd[2:6])
            self.client.log(f"[PUSH] receives push frame. service:{_serviceType}", True)
            if _pushType in [0, 2]:
                _pushPayload = _dd[6:]

                if _pushType == 2:
                    # SEND ACK FOR PUSHES
                    _PushAck = (
                        bytes([1] + [_serviceType]) + struct.pack("!i", _pushId) + b""
                    )
                    _DATA = struct.pack("!H", len(_PushAck)) + bytes([4]) + _PushAck
                    self.conn.send_data(stream_id=1, data=_DATA, end_stream=False)
                    self.client.log(
                        f"[PUSH] send push ack. service:{_serviceType}", True
                    )

                # Callback
                self.manager.OnPushResponse(_serviceType, _pushId, _pushPayload)
            else:
                raise NotImplementedError(f"push type not Implemented: {_pushType}")
        else:
            raise NotImplementedError(
                f"PUSH not Implemented: type:{_dt}, payloads:{_dd[:30]}, len:{len(_dd)}"
            )

    def close(self):
        """Close conn."""
        self._closed = True
        # write close packet
        self.conn.close_connection()
        self.send()
        # close the socket
        self.writer.shutdown(socket.SHUT_RDWR)
        self.writer.close()



# -*- coding: utf-8 -*-

class TestService(object):
    
    def __init__(self):
        pass

    def testRequest(self, m_name: str, m_args: list, api_path: str, req_thrift_type: int = 3, res_thrift_type: int = 3):
        """
        - m_name:
            func name, eg. fetchOps
        - m_args:
            func args for DummyProtocol, eg:
            [
                [12, 1, [
                    [11, 1, 'im string']
                ]]
            ]
        - api_path:
            request path, eg. /C5
        """
        params = m_args
        sqrd = self.generateDummyProtocol(m_name, params, req_thrift_type)
        return self.postPackDataAndGetUnpackRespData(api_path ,sqrd, res_thrift_type)


# -*- coding: utf-8 -*-
import time
import json
import uuid

class PrimaryAccountInitService(object):

    def __init__(self):
        self.register_headers  = self.server.Headers
        self.uuid = uuid.uuid4().hex
    
    def setPrimaryUuid(self, uuid):
        self.uuid = uuid
        
    def openPrimarySession(self):
        params = [
            [12, 1, [
                [13, 1, [11, 11, []]],
            ]]
        ]
        sqrd = self.generateDummyProtocol('openSession', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def getCountryInfo(self, authSessionId, simCard=None):
        params = [
            [11, 1, authSessionId],
            # [12, 11, [
                # [11, 1, countryCode],
                # [11, 2, hni],
                # [11, 3, carrierName],
            # ]]
        ]
        sqrd = self.generateDummyProtocol('getCountryInfo', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def getPhoneVerifMethod(self, authSessionId, phoneNumber, countryCode, deviceModel="SM-N950F"):
        _headers = {
            'x-lpqs': "/acct/pais/v1"
        }
        a = self.encHeaders(_headers)
        sqrd = [128, 1, 0, 1] + self.getStringBytes('getPhoneVerifMethod') + [0, 0, 0, 0]
        sqrd += [11, 0, 1] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 2]
        sqrd += [11, 0, 1] + self.getStringBytes(self.uuid)
        sqrd += [11, 0, 2] + self.getStringBytes(deviceModel)
        sqrd += [0]
        sqrd += [12, 0, 3]
        sqrd += [11, 0, 1] + self.getStringBytes(phoneNumber)
        sqrd += [11, 0, 2] + self.getStringBytes(countryCode)
        sqrd += [0, 0]
        sqr_rd = a + sqrd
        _data = bytes(sqr_rd)
        data = self.encData(_data)
        res = self.server.postContent('https://ga2.line.naver.jp', data=data, headers=self.register_headers)
        data = self.decData(res.content)
        return self.tryReadData(data)
        
    def sendPinCodeForPhone(self, authSessionId, phoneNumber, countryCode, deviceModel="SM-N950F"):
        _headers = {
            'x-lpqs': "/acct/pais/v1"
        }
        a = self.encHeaders(_headers)
        sqrd = [128, 1, 0, 1] + self.getStringBytes('sendPinCodeForPhone') + [0, 0, 0, 0]
        sqrd += [11, 0, 1] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 2]
        sqrd += [11, 0, 1] + self.getStringBytes(self.uuid)
        print(f"UUID: {self.uuid}")
        sqrd += [11, 0, 2] + self.getStringBytes(deviceModel)
        sqrd += [0]
        sqrd += [12, 0, 3]
        sqrd += [11, 0, 1] + self.getStringBytes(phoneNumber)
        sqrd += [11, 0, 2] + self.getStringBytes(countryCode)
        sqrd += [0]
        sqrd += [8, 0, 4] + self.getIntBytes(2)
        sqrd += [0]
        sqr_rd = a + sqrd
        _data = bytes(sqr_rd)
        data = self.encData(_data)
        res = self.server.postContent('https://ga2.line.naver.jp', data=data, headers=self.register_headers)
        data = self.decData(res.content)
        return self.tryReadData(data)
        
    def verifyPhone(self, authSessionId, phoneNumber, countryCode, pinCode, deviceModel="SM-N950F"):
        _headers = {
            'x-lpqs': "/acct/pais/v1"
        }
        a = self.encHeaders(_headers)
        sqrd = [128, 1, 0, 1] + self.getStringBytes('verifyPhone') + [0, 0, 0, 0]
        sqrd += [11, 0, 1] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 2]
        sqrd += [11, 0, 1] + self.getStringBytes(self.uuid)
        sqrd += [11, 0, 2] + self.getStringBytes(deviceModel)
        sqrd += [0]
        sqrd += [12, 0, 3]
        sqrd += [11, 0, 1] + self.getStringBytes(phoneNumber)
        sqrd += [11, 0, 2] + self.getStringBytes(countryCode)
        sqrd += [0]
        sqrd += [11, 0, 4] + self.getStringBytes(pinCode)
        sqrd += [0]
        sqr_rd = a + sqrd
        _data = bytes(sqr_rd)
        data = self.encData(_data)
        res = self.server.postContent('https://ga2.line.naver.jp', data=data, headers=self.register_headers)
        data = self.decData(res.content)
        return self.tryReadData(data)
        
    def validateProfile(self, authSessionId, displayName):
        params = [
            [11, 1, authSessionId],
            [11, 2, displayName]
        ]
        sqrd = self.generateDummyProtocol('validateProfile', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def exchangeEncryptionKey(self, authSessionId, publicKey, nonce, authKeyVersion=1):
        params = [
            [11, 1, authSessionId],
            [12, 2, [
                [8, 1, authKeyVersion],
                [11, 2, publicKey],
                [11, 3, nonce],
            ]]
        ]
        sqrd = self.generateDummyProtocol('exchangeEncryptionKey', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def setPassword(self, authSessionId, cipherText, encryptionKeyVersion=1):
        params = [
            [11, 1, authSessionId],
            [12, 2, [
                [8, 1, encryptionKeyVersion],
                [11, 2, cipherText],
            ]]
        ]
        sqrd = self.generateDummyProtocol('setPassword', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def registerPrimaryUsingPhone(self, authSessionId):
        _headers = {
            'x-lpqs': "/acct/pais/v1"
        }
        a = self.encHeaders(_headers)
        sqrd = [128, 1, 0, 1] + self.getStringBytes('registerPrimaryUsingPhone') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(authSessionId)
        sqrd += [0]
        sqr_rd = a + sqrd
        _data = bytes(sqr_rd)
        data = self.encData(_data)
        res = self.server.postContent('https://ga2.line.naver.jp', data=data, headers=self.register_headers)
        data = self.decData(res.content)
        return self.tryReadData(data)
        
    def getPhoneVerifMethodV2(self, authSessionId, phoneNumber, countryCode, deviceModel="SM-N950F"):
        params = [
            [12, 1, [
                [11, 1, authSessionId],
                [12, 2, [
                    [11, 1, self.uuid],
                    [11, 2, deviceModel]
                ]],
                [12, 3, [
                    [11, 1, phoneNumber],
                    [11, 2, countryCode]
                ]],
            ]]
        ]
        sqrd = self.generateDummyProtocol('getPhoneVerifMethodV2', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def requestToSendPhonePinCode(self, authSessionId, phoneNumber, countryCode, verifMethod=1):
        params = [
            [12, 1, [
                [11, 1, authSessionId],
                [12, 2, [
                    [11, 1, phoneNumber],
                    [11, 2, countryCode]
                ]],
                [8, 3, verifMethod],
            ]]
        ]
        sqrd = self.generateDummyProtocol('requestToSendPhonePinCode', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def verifyPhonePinCode(self, authSessionId, phoneNumber, countryCode, pinCode):
        params = [
            [12, 1, [
                [11, 1, authSessionId],
                [12, 2, [
                    [11, 1, phoneNumber],
                    [11, 2, countryCode]
                ]],
                [11, 3, pinCode],
            ]]
        ]
        sqrd = self.generateDummyProtocol('verifyPhonePinCode', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def verifyAccountUsingPwd(self, authSessionId, identifier, countryCode, cipherText):
        params = [
            [12, 1, [
                [11, 1, authSessionId],
                [12, 2, [
                    [8, 1, 1], # type
                    [11, 2, identifier],
                    [11, 3, countryCode]
                ]],
                [12, 3, [
                    [8, 1, 1], # encryptionKeyVersion
                    [11, 2, cipherText]
                ]],
            ]]
        ]
        sqrd = self.generateDummyProtocol('verifyAccountUsingPwd', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def registerPrimaryUsingPhoneWithTokenV3(self, authSessionId):
        params = [
            [11, 2, authSessionId]
        ]
        sqrd = self.generateDummyProtocol('registerPrimaryUsingPhoneWithTokenV3', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def registerPrimaryWithTokenV3(self, authSessionId):
        params = [
            [11, 2, authSessionId]
        ]
        sqrd = self.generateDummyProtocol('registerPrimaryUsingPhoneWithTokenV3', params, 3)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 3, headers=self.register_headers)
        
    def lookupAvailableEap(self, authSessionId):
        params = [
            [11, 1, authSessionId]
        ]
        params = [
            [12, 1, params]
        ]
        sqrd = self.generateDummyProtocol('lookupAvailableEap', params, 4)
        return self.postPackDataAndGetUnpackRespData("/acct/pais/v1" ,sqrd, 4, headers=self.register_headers)


# -*- coding: utf-8 -*-
"""
CHRLINE Timeline Story.

Created by DeachSword/YinMo
Created Time: 2022-09-19
"""

import time


class Story(object):
    def __init__(self, a: int):
        self.a = a

    @staticmethod
    def CreateStoryContent(**kwargs):
        contentBody = {
            "sourceType": kwargs["sourceType"],
            "contentType": kwargs["contentType"],
        }
        medias = []
        for media in kwargs["medias"]:
            medias.append(media.get())
        contentBody["media"] = medias
        if "referrerType" in kwargs is not None:
            contentBody["referrerType"] = kwargs["referrerType"]
        if "meta" in kwargs is not None:
            contentBody["meta"] = kwargs["meta"]
        shareInfoBody = {"shareType": kwargs["shareType"]}
        if "shareGroupIds" in kwargs is not None:
            shareInfoBody["shareGroupIds"] = kwargs["shareGroupIds"]
        return {"content": contentBody, "shareInfo": shareInfoBody}

    @staticmethod
    def DeleteStoryContent(**kwargs):
        return {"contentId": kwargs["contentId"]}

    @staticmethod
    def GetStoryList(**kwargs):
        userInfosBody = []
        clickedUserInfo = {"userMid": kwargs["userMid"], "tsId": ""}
        for mid in kwargs["userMids"]:
            info = {
                "userMid": mid,
                # guideId": guideId,
                "tsId": "",
            }
            userInfosBody.append(info)
        return {"userInfos": userInfosBody, "clickedUserInfo": clickedUserInfo}

    @staticmethod
    def GetStory(**kwargs):
        return {"userMid": kwargs["userMid"]}

    @staticmethod
    def GetStoryContent(**kwargs):
        return {"contentId": kwargs["contentId"]}

    @staticmethod
    def ReadStoryContent(**kwargs):
        return {
            # "userMid": kwargs['userMid'],
            "contentId": kwargs["contentId"],
            "createdTime": int(time.time() * 1000),
            "tsId": kwargs["tsId"],
        }

    @staticmethod
    def LikeStoryContent(**kwargs):
        return {
            "contentId": kwargs["contentId"],
            "tsId": kwargs["tsId"],
            "like": kwargs["like"],
            "likeType": kwargs["likeType"],
        }

    @staticmethod
    def StoryContentLikeList(**kwargs):
        baseBody = {
            # "userMid": kwargs['userMid'],
            "contentId": kwargs["contentId"],
            "include": kwargs["include"],
            "likeType": kwargs["likeType"],
            "size": kwargs["size"],
        }
        if "scrollId" in kwargs:
            baseBody["scrollId"] = kwargs["scrollId"]
        return baseBody

    @staticmethod
    def ReportStoryContent(**kwargs):
        return {
            "contentId": kwargs["contentId"],
            "type": kwargs["type"],
        }

    @staticmethod
    def SendStoryMessage(**kwargs):
        return {
            "contentId": kwargs["contentId"],
            "to": kwargs["to"],
            "message": kwargs["message"],
        }

    @staticmethod
    def GetStoryGuild(**kwargs):
        return {
            "guideId": kwargs["guideId"],
        }

    @staticmethod
    def NotifyStoryGuild(**kwargs):
        return {
            # "userMid": kwargs['userMid'],
            "guideId": kwargs["guideId"],
            "guideContentId": kwargs["guideContentId"],
            "notifyType": kwargs["notifyType"],
            "value": kwargs["value"],
            "tsId": kwargs["tsId"],
        }

    @staticmethod
    def GetOwnerStoryContentList(**kwargs):
        baseBody = {
            "order": kwargs["order"],
            "size": kwargs["size"],
        }
        if "lastContentId" in kwargs:
            baseBody["lastContentId"] = kwargs["lastContentId"]
        return baseBody

    @staticmethod
    def GetOwnerStoryContentViewList(**kwargs):
        return {
            "contentId": kwargs["contentId"],
            "order": kwargs["order"],
            "size": kwargs["size"],
        }

    @staticmethod
    def GetOwnerStoryContentViewMoreList(**kwargs):
        return {
            "lastContentId": kwargs["lastContentId"],
            "direction": kwargs["direction"],
            "order": kwargs["order"],
            "size": kwargs["size"],
        }

    @staticmethod
    def GetRecentStoryList(**kwargs):
        baseBody = {
            "lastRequestTime": kwargs["lastRequestTime"],
        }
        if "lastTimelineVisitTime" in kwargs:
            baseBody["lastTimelineVisitTime"] = kwargs["lastTimelineVisitTime"]
        return baseBody

    @staticmethod
    def GetStoryContentViewerList(**kwargs):
        baseBody = {
            "contentId": kwargs["contentId"],
            "size": kwargs["size"],
        }
        if "scrollId" in kwargs:
            baseBody["scrollId"] = kwargs["scrollId"]
        return baseBody

    @staticmethod
    def GetStoryChallengeeContentList(**kwargs):
        baseBody = {
            "guideId": kwargs["guideId"],
            "direction": kwargs["direction"],
            "size": kwargs["size"],
        }
        if "lastContentSeq" in kwargs:
            baseBody["lastContentSeq"] = kwargs["lastContentSeq"]
        if "popularContentIds" in kwargs:
            baseBody["popularContentIds"] = kwargs["popularContentIds"]
        return baseBody

    @staticmethod
    def GetNewStory(**kwargs):
        baseBody = {
            "newStoryTypes": [
                "WRITE",
                "MY",
                "GUIDE",
                "CHALLENGE",
                "USER",
                "ARCHIVE",
                "LIVE",
                "INVALID",
            ],
        }
        if "lastTimelineVisitTime" in kwargs:
            baseBody["lastTimelineVisitTime"] = kwargs["lastTimelineVisitTime"]
        return baseBody

    def GetApiPath(self, path):
        return f"/st/api/v{self.a}/{path}"


class StoryMedia(object):
    def __init__(self, oid: str, ohash: str, oType: str):
        self.oid = oid
        self.ohash = ohash
        self.oType = oType

    def get(self):
        return {
            "oid": self.oid,
            "service": "story",
            "sid": "st",
            "hash": self.ohash,
            "extra": {"playtime": 99999999999999999999999999},
            "mediaType": self.oType,
        }



# -*- coding: utf-8 -*-


class AccessTokenRefreshService(object):

    def __init__(self):
        pass

    def refreshAccessToken(self, refreshToken):
        METHOD_NAME = "refresh"
        params = [[12, 1, [[11, 1, refreshToken]]]]
        sqrd = self.generateDummyProtocol(METHOD_NAME, params, 3)
        return self.postPackDataAndGetUnpackRespData(
            "/EXT/auth/tokenrefresh/v1",
            sqrd,
            3,
            readWith=f"{__class__.__name__}.{METHOD_NAME}")

    def reportRefreshedAccessToken(self, refreshToken):
        METHOD_NAME = "reportRefreshedAccessToken"
        params = [[12, 1, [[11, 1, refreshToken]]]]
        sqrd = self.generateDummyProtocol(METHOD_NAME, params, 3)
        return self.postPackDataAndGetUnpackRespData(
            "/EXT/auth/tokenrefresh/v1",
            sqrd,
            3,
            readWith=f"{__class__.__name__}.{METHOD_NAME}")



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from CHRLINE import CHRLINE


class BaseService:
    def __init__(self):
        pass


class BaseServiceStruct:
    @staticmethod
    def BaseRequest(request: list):
        return [[12, 1, request]]

    @staticmethod
    def SendRequestByName():
        raise NotImplementedError


class BaseServiceHandler:
    def __init__(self, client: "CHRLINE") -> None:
        self.cl = client
